Attribute VB_Name = "Validation"
'#########################################################################
' Copyright  Center for Disease Control and Prevention, 1998.
'
' File Name: Validation.bas
'
' Description:
'
' Original Author: Setumo Mohapi
'
' Date Created:
' $Archive: $
' $Revision: 1.18 $
' $Date: 2004/11/19 13:57:46 $
' $Author: ldoucett $
'#########################################################################
Option Explicit

'#########################################################################
' Subroutine Name: ValidateSubdivision
'
' Description: The subroutine validates that the subdivision being used
' exists in the database. Exceptions are thrown if it does not or if
' there are multiple entries in the database.
'
' Arguments:
'           aSubdivision - Subdivision being validated.
'           aConnection - An open connection object to the database.
'
' Exceptions:
'           VM_SUBDIVISION_ERROR
'           VM_DUPLICATE_SUBDIVISION
'           VM_UNKNOWN_SUBDIVISION
'
'#########################################################################
Public Sub ValidateSubdivision(aSubdivision As String, aConnection As ADODB.Connection)
    On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateSubdivision()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    
    Set recordset = CreateObject("ADODB.RecordSet")
    
    query = "SELECT count(*) FROM subdiv WHERE ltrim(rtrim(subdiv)) = '" & aSubdivision & "'"
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_SUBDIVISION_ERROR_NUMBER, _
                    VM_SUBDIVISION_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount <> 1 Then
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_SUBDIVISION_NUMBER, _
            VM_DUPLICATE_SUBDIVISION_MESSAGE
    
        ElseIf recordCount = 0 Then
            RaiseError VM_UNKNOWN_SUBDIVISION_NUMBER, _
                VM_UNKNOWN_SUBDIVISION_MESSAGE
        
        Else
            RaiseError VM_SUBDIVISION_ERROR_NUMBER, _
                    VM_SUBDIVISION_ERROR_MESSAGE
            
        End If
        
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateSubdivision()", vbLogEventTypeInformation

Exit Sub

ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateSubdivision() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Subroutine Name: ValidateShippingAddress
'
' Description: Validates that that Address1, City and State input paramenters
' are null or empty strings. Also checks that the ZipCode parameter is
' exists in the database. The Address2 parameter is set to the string NONE_STRING
' if it is null or it is an empty. Function assumes that the connection parameter
' is a valid reference and is open.
'
' Arguments:
'   anAddress1 : The first line of the address.
'   anAddress2 : Second line of the address.
'   aCity : The city
'   aZipCode : 5 digit zip code. This argument is validated against correct format
'   aState : The state
'   aConnection : An open connection object
'
' Exceptions:   VM_NULL_SHIPPING_ADDRESS1
'               VM_NULL_SHIPPING_CITY
'               VM_NULL_SHIPPING_STATE
'
'#########################################################################

Public Sub ValidateShippingAddress(anAddress1 As String, anAddress2 As String, aCity As String, aZipCode As String, aState As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateShippingAddress()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim zipCode As String
    
    Set recordset = CreateObject("ADODB.Recordset")

    If ValidateNullString(anAddress1) = True Then
        RaiseError VM_NULL_SHIPPING_ADDRESS1_NUMBER, _
                    VM_NULL_SHIPPING_ADDRESS1_MESSAGE
    End If
            
            
    If ValidateNullString(aCity) = True Then
        RaiseError VM_NULL_SHIPPING_CITY_NUMBER, _
                    VM_NULL_SHIPPING_CITY_MESSAGE
    End If
    
    If ValidateNullString(aState) = True Then
        RaiseError VM_NULL_SHIPPING_STATE_NUMBER, _
                    VM_NULL_SHIPPING_STATE_MESSAGE
    End If
    ' Mark Fleming 02-05-2007 - Temporarily comment out zip code check
    ' App.LogEvent "Before calling ValidateZipCode", vbLogEventTypeInformation
    ' ValidateZipCode aZipCode, aCity, aConnection
    ' App.LogEvent "After calling ValidateZipCode", vbLogEventTypeInformation

    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateShippingAddress()", vbLogEventTypeInformation
    
    Exit Sub
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    
    App.LogEvent "Problem: In ValidateShippingAddress() " & desc, vbLogEventTypeError
    
    Err.Raise code, "VACMAN DLL", desc
    
End Sub



'#########################################################################
' Subroutine Name: ValidateZipCode
'
' Description: The subroutine checks whether the first five characters of the
' zip code exists in the database.
'
' Arguments: aZipCode - Zip code to validate
'           aConnection - An open connection object to a database
'
' Exceptions:  VM_ZIP_CODE_RECORD_COUNT_ERROR
'               VM_DUPLICATE_ZIP_CODE
'               VM_ZIP_CODE_NOT_FOUND
'
'#########################################################################

Public Sub ValidateZipCode(aZipCode As String, aCity As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateZipCode()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim zipCode As String
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    zipCode = Left(Trim(aZipCode), ZIP_CODE_DIGITS)
    query = "SELECT count(*) FROM zipcodes WHERE ltrim(rtrim(zip)) = '" & zipCode & "'"
    query = query & " and ltrim(rtrim(city)) = '" & aCity & "'"
    
    App.LogEvent "ValidateZipCodeQuery is :" & query, vbLogEventTypeInformation

    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount

    App.LogEvent "After calling ValidateZipCodeQuery", vbLogEventTypeInformation

     If rowCount < 1 Then
         RaiseError VM_ZIP_CODE_RECORD_COUNT_ERROR_NUMBER, _
                    VM_ZIP_CODE_RECORD_COUNT_ERROR_MESSAGE
     Else
        'App.LogEvent "ValidateZipCodeQuery result is not < 1 :", vbLogEventTypeInformation
     
        recordCount = recordset.Fields.Item(0)
        

        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_ZIP_CODE_NUMBER, _
                        VM_DUPLICATE_ZIP_CODE_MESSAGE
                        
        ElseIf recordCount = 0 Then
             RaiseError VM_ZIP_CODE_NOT_FOUND_NUMBER, _
                        VM_ZIP_CODE_NOT_FOUND_MESSAGE
                        
        ElseIf recordCount < 0 Then
            RaiseError VM_ZIP_CODE_RECORD_COUNT_ERROR_NUMBER, _
                     VM_ZIP_CODE_RECORD_COUNT_ERROR_MESSAGE
                     
        End If
    
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateZipCode()", vbLogEventTypeInformation
    
    Exit Sub

ErrorHandler:
    'App.LogEvent "Problem: In ValidateZipCode()before anything ", vbLogEventTypeError
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateZipCode() " & desc, vbLogEventTypeError
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Err.Raise code, "VACMAN DLL", desc

End Sub



'#########################################################################
' Subroutine Name:  ValidateOrderDate
'
' Description:      Validate that the order date is earlier than today's date
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateOrderDate(anOrderDate As String)
    App.LogEvent "Begin: ValidateOrderDate()", vbLogEventTypeInformation

    Dim currentDate As Date
    Dim orderDate As Date
    
    orderDate = DateValue(anOrderDate)
    currentDate = Date
    If currentDate < orderDate Then
        App.LogEvent "End: ValidateOrderDate()", vbLogEventTypeInformation
    
        RaiseError VM_ORDER_DATE_ERROR_NUMBER, _
                    VM_ORDER_DATE_ERROR_MESSAGE
        
    End If
    App.LogEvent "End: ValidateOrderDate()", vbLogEventTypeInformation

End Sub

'#########################################################################
' Subroutine Name:  ValidateVaccAdminDate
'
' Description:      Validate that the admin date is earlier than today's date
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateVaccAdminDate(anOrderDate As String)
    App.LogEvent "Begin: ValidateVaccAdminDate()", vbLogEventTypeInformation

    Dim currentDate As Date
    Dim orderDate As Date
    
    orderDate = DateValue(anOrderDate)
    currentDate = Date
    
    If currentDate < orderDate Then
        App.LogEvent "End: ValidateVaccAdminDate()", vbLogEventTypeInformation
    
        RaiseError VM_VACC_ADMIN_REPORT_DATE_ERROR_NUMBER, _
                    VM_VACC_ADMIN_REPORT_DATE_ERROR_MESSAGE
        
    End If
    App.LogEvent "End: ValidateVaccAdminDate()", vbLogEventTypeInformation

End Sub

'#########################################################################
' Subroutine Name:  ValidateShipmentDate
'
' Description:      Validate that the ship date is earlier than today's date
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateShipmentDate(anOrderDate As String, aShipmentDate As String)
    App.LogEvent "Begin: ValidateShipmentDate()", vbLogEventTypeInformation
    
    Dim shipmentDate
    Dim orderDate
    orderDate = DateValue(anOrderDate)
    shipmentDate = DateValue(aShipmentDate)
    
    If shipmentDate < orderDate Then
        App.LogEvent "End: ValidateShipmentDate()", vbLogEventTypeInformation
    
        RaiseError VM_SHIPMENT_DATE_ERROR_NUMBER, _
                    VM_SHIPMENT_DATE_ERROR_MESSAGE
        
    End If
    App.LogEvent "End: ValidateShipmentDate()", vbLogEventTypeInformation

End Sub

'#########################################################################
' Subroutine Name:  ValidateDepotPIN
'
' Description:      Validate the existence of the depot pin
'
' Arguments:
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  Sector is a number value. Removed
'                                         the trim functions.
'#########################################################################
Public Sub ValidateDepotPIN(aPIN As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDepotPIN()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim PIN As String
     
    Set recordset = CreateObject("ADODB.Recordset")
        
    PIN = UCase(aPIN)
    
    If Len(PIN) > MAX_PIN_SIZE Then
        RaiseError VM_MAX_PIN_SIZE_VIOLATION_NUMBER, _
                    VM_MAX_PIN_SIZE_VIOLATION_MESSAGE
    End If
    
    If IsAlphaNumeric(PIN) = False Then
        RaiseError VM_INVALID_DEPOT_PIN_TYPE_NUMBER, _
                    VM_INVALID_DEPOT_PIN_TYPE_MESSAGE
    End If
    
    query = "SELECT count(*) "
    query = query & "FROM provider "
    query = query & "WHERE ltrim(rtrim(active)) = 'Y' "
    ''' R1272 query = query & "AND ltrim(rtrim(sector)) "
    query = query & "AND sector "
    query = query & "BETWEEN " & DEPOT_SECTOR_MIN & " AND " & DEPOT_SECTOR_MAX & " "
    query = query & "AND ltrim(rtrim(pin)) = '" & PIN & "'"

    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount < 1 Then
         RaiseError VM_DEPOT_PIN_RECORD_COUNT_ERROR_NUMBER, _
                    VM_DEPOT_PIN_RECORD_COUNT_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
    
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_DEPOT_PINS_NUMBER, _
                        VM_DUPLICATE_DEPOT_PINS_MESSAGE
        ElseIf recordCount < 1 Then
            RaiseError VM_DEPOT_PIN_RECORD_COUNT_ERROR_NUMBER, _
                        VM_DEPOT_PIN_RECORD_COUNT_ERROR_MESSAGE
        End If
    End If
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDepotPIN()", vbLogEventTypeInformation
    Exit Sub
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDepotPIN() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Sub
'#########################################################################
' Subroutine Name:  ValidateNewProviderPIN
'
' Description: The subroutine checks that the new pin is not
' one of the reserved pins.
'
' Arguments: aPIN - The new pin to validate
'
' Exceptions: VM_CANNOT_ASSIGN_RESERVED_PIN
'
'#########################################################################
Public Sub ValidateNewProviderPIN(aPIN As String)

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateNewProviderPIN()", vbLogEventTypeInformation
    
    If StrComp(aPIN, RESERVED_PIN_1) = 0 Or StrComp(aPIN, RESERVED_PIN_2) = 0 Then
        RaiseError VM_CANNOT_ASSIGN_RESERVED_PIN_NUMBER, _
                    VM_CANNOT_ASSIGN_RESERVED_PIN_MESSAGE
    End If
    
    If StrComp(aPIN, ZERO_PIN) = 0 Then
        RaiseError VM_ZERO_PIN_ERROR_NUMBER, _
                    VM_ZERO_PIN_ERROR_MESSAGE
    End If
    App.LogEvent "End: ValidateNewProviderPIN()", vbLogEventTypeInformation
Exit Sub

ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateNewProviderPIN() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Function Name: ValidateProviderPIN
'
' Description: If the aPIN parameter is not null or an empty string
' the function checks whether the pin exists. Exceptions are raised if the input pin
' is null or is an empty or if there are duplicate entries in the
' database.
'
' Arguments:
'       aPIN - The pin of the provider being validated.
'       aValidateActiceFlag - indication for validating the active status of the practice
'       aConnection - An open connection object.
'
' Returns: If the pin exists, then the function returns TRUE.
' If the pin does not exist then the function returns FALSE.
' The function assumes that the connection input is open
'
' Exceptions:   VM_NULL_PROVIDER_PIN
'               VM_PROVIDER_PIN_RECORD_COUNT_ERROR
'               VM_DUPLICATE_PROVIDER_PINS
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  Sector is a number value. Removed
'                                         the trim functions.
'#########################################################################
Public Function ValidateProviderPIN(aPIN As String, aValidateActiceFlag As Boolean, aConnection As ADODB.Connection) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateProviderPIN()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim PIN As String
    
    
    Set recordset = CreateObject("ADODB.Recordset")

    PIN = UCase(aPIN)
        
    If Len(PIN) > MAX_PIN_SIZE Then
        RaiseError VM_MAX_PIN_SIZE_VIOLATION_NUMBER, _
                    VM_MAX_PIN_SIZE_VIOLATION_MESSAGE
        
    End If
    
    
    If IsAlphaNumeric(PIN) = False Then
        RaiseError VM_INVALID_PROVIDER_PIN_TYPE_NUMBER, _
                    VM_INVALID_PROVIDER_PIN_TYPE_MESSAGE
    End If
    
    
    If True = ValidateNullString(PIN) Then
        RaiseError VM_NULL_PROVIDER_PIN_NUMBER, _
                VM_NULL_PROVIDER_PIN_MESSAGE
    End If
            
 
    ''' R1272 query = "SELECT count(*) FROM provider WHERE ltrim(rtrim(Sector)) BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX & " AND ltrim(rtrim(pin)) = '" & PIN & "'"
    query = "SELECT count(*) FROM provider WHERE Sector BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX & " AND ltrim(rtrim(pin)) = '" & PIN & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
        RaiseError VM_PROVIDER_PIN_RECORD_COUNT_ERROR_NUMBER, _
                    VM_PROVIDER_PIN_RECORD_COUNT_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
        
        If recordCount = 0 Then
             ValidateProviderPIN = False
             
        ElseIf recordCount = 1 Then
            If aValidateActiceFlag = True Then
                ValidateProviderActiveStatus aPIN, aConnection
            End If
            ValidateProviderPIN = True
    
        ElseIf recordCount > 1 Then
            RaiseError VM_DUPLICATE_PROVIDER_PINS_NUMBER, _
                        VM_DUPLICATE_PROVIDER_PINS_MESSAGE
        
        Else
            RaiseError VM_PROVIDER_PIN_RECORD_COUNT_ERROR_NUMBER, _
                    VM_PROVIDER_PIN_RECORD_COUNT_ERROR_MESSAGE
        
        End If
    
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateProviderPIN()", vbLogEventTypeInformation
      
    Exit Function
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateProviderPIN() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Subroutine Name: ValidateProviderActiveStatus
'
' Description: The subroutine checks if the given pin belongs to an active provider.
' An exception is raised if the provider record for the given pin is not marked as active.
'
' Arguments:
'       aPIN - The pin of the provider being validated.
'       aConnection - An open connection object.
'
' Exceptions:   VM_PROVIDER_PIN_RECORD_COUNT_ERROR
'               VM_INACTIVE_PROVIDER_ERROR
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  Sector is a number value. Removed
'                                         the trim functions.
'#########################################################################
Public Sub ValidateProviderActiveStatus(aPIN As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateProviderActiveStatus()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordValue As String
    Dim rowCount As Long
    Dim PIN As String

    Set recordset = CreateObject("ADODB.Recordset")
    
    PIN = UCase(aPIN)
    
    ''' R1272 query = "SELECT active FROM provider WHERE ltrim(rtrim(Sector)) BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX & " AND ltrim(rtrim(pin)) = '" & PIN & "'"
    query = "SELECT active FROM provider WHERE Sector BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX & " AND ltrim(rtrim(pin)) = '" & PIN & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
        RaiseError VM_PROVIDER_PIN_RECORD_COUNT_ERROR_NUMBER, _
                    VM_PROVIDER_PIN_RECORD_COUNT_ERROR_MESSAGE
    Else
        If IsNull(recordset.Fields.Item(0)) = False Then
            recordValue = recordset.Fields.Item(0)
        Else
            recordValue = "N"
        End If
        
        If Not (StrComp("Y", UCase(recordValue)) = 0) Then
            RaiseError VM_INACTIVE_PROVIDER_ERROR_NUMBER, _
                        VM_INACTIVE_PROVIDER_ERROR_MESSAGE
        End If
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateProviderActiveStatus()", vbLogEventTypeInformation
Exit Sub

ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateProviderActiveStatus() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Sub

'#########################################################################
' Function Name: ValidateVaccineActiveStatus
'
' Description: Function checks whether the given vaccine is marked active
' in the database.
'
' Arguments:
'       aVaccine - The vaccine being validated.
'       aConnection - Open connection object to the database.
'
' Returns : True if vaccine is active and False otherwise
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id      Date          Author          Description
' ----------------------------------------------------------------------------
' IMMPACT-529    06/08/2007    Mark Fleming    Change from using vaccref table
'                                              to pvaccref and Nonfedvac tables.
'                                              These new tables replaced vaccref
'                                              in Vacman 4.0
'#########################################################################
Public Function ValidateVaccineActiveStatus(aVaccine As String, aConnection As ADODB.Connection) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateVaccineActiveStatus()", vbLogEventTypeInformation
    Dim RetVal As Boolean
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    
    Set recordset = CreateObject("ADODB.RecordSet")
    
    query = "SELECT count(*) FROM pvaccref WHERE ltrim(rtrim(vaccine)) == '" & aVaccine & "' AND locactiv = .F."
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
'    If rowCount <> 1 Then
'          RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
'                    VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
'    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount >= 1 Then
        RetVal = True
    Else
        RetVal = False
    End If
    
    ValidateVaccineActiveStatus = RetVal
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateVaccineActiveStatus()", vbLogEventTypeInformation

Exit Function
ErrorHandler:
     With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateVaccineActiveStatus() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Subroutine Name: ValidateDoseSeriesNumber
'
' Description: Subroutine validates that the dose in series number
' is between 0 and the maximum number for the given vaccine. The
' maximum number is stored in the vaccref table.
'
'
' Arguments:
'       aDoseInSeries - The series number being validated.
'       aVaccine - Vaccine whose series numbers are being validated
'       aConnection - Open connection object to the database.
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id      Date          Author          Description
' ----------------------------------------------------------------------------
' IMMPACT-529    06/08/2007    Mark Fleming    Change from using vaccref table
'                                              to pvaccref and Nonfedvac tables.
'                                              These new tables replaced vaccref
'                                              in Vacman 4.0
' IMMPACT-1059   03/13/2008    Matt Martin     Change from using pvaccref to vaccref
'                                              tables (MMR-V)
'#########################################################################
Public Sub ValidateDoseSeriesNumber(aDoseInSeries As String, aVaccine As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDoseSeriesNumber()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim pmdValue As String
    Dim rowCount As Long
    Dim recordValue As Long
    Dim maxDoseNumber As String
    
    Set recordset = CreateObject("ADODB.Recordset")

    If CLng(aDoseInSeries) <= MIN_DOSE_IN_SERIES_NUMBER Then
        RaiseError VM_MIN_DOSE_IN_SERIES_VIOLATION_NUMBER, _
                VM_MIN_DOSE_IN_SERIES_VIOLATION_MESSAGE
    End If

    query = "SELECT MAX( maxno) FROM vaccref WHERE ltrim(rtrim(vaccine)) == '" & aVaccine & "' AND locactiv== 'Y'"
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
'    If rowCount <> 1 Then
    'technical error
'        RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
'                    VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
'    Else
        If IsNull(recordset.Fields.Item(0)) = False Then
            recordValue = recordset.Fields.Item(0)
        Else
            recordValue = 0
        End If
        
        'recordValue = recordset.Fields.Item(0)
        
        If (CLng(aDoseInSeries) > recordValue) Then
            RaiseError VM_MAX_DOSE_IN_SERIES_VIOLATION_NUMBER, _
                VM_MAX_DOSE_IN_SERIES_VIOLATION_MESSAGE
        End If
'    End If

    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDoseSeriesNumber()", vbLogEventTypeInformation
Exit Sub

ErrorHandler:
     With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDoseSeriesNumber() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Subroutine Name: ValidateDoseSeriesNumberPvaccref
'
' Description: Subroutine validates that the dose in series number
' is between 0 and the maximum number for the given vaccine. The
' maximum number is stored in the pvaccref table.
'
'
' Arguments:
'       aDoseInSeries - The series number being validated.
'       aVaccine - Vaccine whose series numbers are being validated
'       aConnection - Open connection object to the database.
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id      Date          Author          Description
' ----------------------------------------------------------------------------

'#########################################################################
Public Sub ValidateDoseSeriesNumberPvaccref(aDoseInSeries As String, aVaccine As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDoseSeriesNumberPvaccref()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim pmdValue As String
    Dim rowCount As Long
    Dim recordValue As Long
    Dim maxDoseNumber As String
    
    Set recordset = CreateObject("ADODB.Recordset")

    If CLng(aDoseInSeries) <= MIN_DOSE_IN_SERIES_NUMBER Then
        RaiseError VM_MIN_DOSE_IN_SERIES_VIOLATION_NUMBER, _
                VM_MIN_DOSE_IN_SERIES_VIOLATION_MESSAGE
    End If

    query = "SELECT MAX( maxno) FROM pvaccref WHERE ltrim(rtrim(vaccine)) == '" & aVaccine & "' AND locactiv=.F."
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
'    If rowCount <> 1 Then
    'technical error
'        RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
'                    VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
'    Else
        If IsNull(recordset.Fields.Item(0)) = False Then
            recordValue = recordset.Fields.Item(0)
        Else
            recordValue = 0
        End If
        
        'recordValue = recordset.Fields.Item(0)
        
        If (CLng(aDoseInSeries) > recordValue) Then
            RaiseError VM_MAX_DOSE_IN_SERIES_VIOLATION_NUMBER, _
                VM_MAX_DOSE_IN_SERIES_VIOLATION_MESSAGE
        End If
'    End If

    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDoseSeriesNumberPvaccref()", vbLogEventTypeInformation
Exit Sub

ErrorHandler:
     With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDoseSeriesNumberPvaccref() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Subroutine Name: ValidateAgainstVaccRefTable
'
' Description: Subroutine validates that the input vaccine exists
' in the database for the provider with the given pin.
' If the provider with aPIN is a private provider
' and if the pmd value of the in vaccref table for the vaccine is "N"
' then it is assumed that the vaccine does not exist for the provider.
' An exception is raised if there are duplicate records in the
' vaccref table with the same vaccine name.
'
' Arguments:
'       aVaccine - The vaccine being validated.
'       aConnection - Open connection object to the database.
'
' Exceptions:   VM_VACC_REF_RECORD_COUNT_ERROR
'               VM_VACCINE_DOES_NOT_EXIST
'               VM_DUPLICATE_VACC_REF
'               VM_PROVIDER_PIN_RECORD_COUNT_ERROR
'               VM_DUPLICATE_PROVIDER_PINS
'
' CHANGE HISTORY
' Request Id     Date          Author          Description
' -------------------------------------------------------------------------------------
' R1352          11/05/2001    Stephen Nelson  Sector is a number value. Removed
'                                              the trim functions.
' IMMPACT-529    06/08/2007    Mark Fleming    Change from using vaccref table
'                                              to pvaccref table.
'                                              These new table replaced vaccref
'                                              in Vacman 4.0.
'                                              Removed aPin as parm and checks
'                                              for pmd column.
'                                              Changed check of query results to
'                                              not return an error on duplicates
'                                              because of contents of pvaccref table.
' IMMPACT-1059   03/13/2008    Matt Martin     Change from using pvaccref table
'                                              to vaccref table (MMR-V)
'######################################################################################
Public Sub ValidateAgainstVaccRefTable( _
                                        aVaccine As String, _
                                        aConnection As ADODB.Connection, _
                                        Optional anActiveCheck As Boolean = True)
    
On Error GoTo ErrorHandler
    App.LogEvent "End: ValidateAgainstVaccRefTable()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim recordset2 As ADODB.recordset
    Dim query As String
    Dim pmdValue As String
    Dim rowCount As Long
    Dim recordCount As Long
        
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM vaccref WHERE ltrim(rtrim(vaccine)) == '" & aVaccine & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
'    If rowCount <> 1 Then
'        'technical error
'        RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
'                    VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
'    Else
        recordCount = recordset.Fields.Item(0)
        
        If recordCount < 1 Then
            If recordCount = 0 Then
                RaiseError VM_VACCINE_DOES_NOT_EXIST_NUMBER, _
                           VM_VACCINE_DOES_NOT_EXIST_MESSAGE
            Else
                'technical error
                RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
                           VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
                        
            End If
                
        End If
        
 '   End If
        
    CloseRecordSet recordset
                        
    
    If anActiveCheck = True Then
        If ValidateVaccineActiveStatus(aVaccine, aConnection) = False Then
            RaiseError VM_INACTIVE_VACCINE_NUMBER, _
                       VM_INACTIVE_VACCINE_MESSAGE
        End If
        
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateAgainstVaccRefTable()", vbLogEventTypeInformation
    
    Exit Sub
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateAgainstVaccRefTable() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Subroutine Name: ValidateAgainstPVaccRefTable
'
' Description: Subroutine validates that the input vaccine exists
' in the database for the provider with the given pin.
' If the provider with aPIN is a private provider
' and if the pmd value of the in pvaccref table for the vaccine is "N"
' then it is assumed that the vaccine does not exist for the provider.
' An exception is raised if there are duplicate records in the
' pvaccref table with the same vaccine name.
'
' Arguments:
'       aVaccine - The vaccine being validated.
'       aConnection - Open connection object to the database.
'
' Exceptions:   VM_VACC_REF_RECORD_COUNT_ERROR
'               VM_VACCINE_DOES_NOT_EXIST
'               VM_DUPLICATE_VACC_REF
'               VM_PROVIDER_PIN_RECORD_COUNT_ERROR
'               VM_DUPLICATE_PROVIDER_PINS
'
' CHANGE HISTORY
' Request Id     Date          Author          Description
' -------------------------------------------------------------------------------------
'######################################################################################
Public Sub ValidateAgainstpVaccRefTable( _
                                        aVaccine As String, _
                                        aConnection As ADODB.Connection, _
                                        Optional anActiveCheck As Boolean = True)
    
On Error GoTo ErrorHandler
    App.LogEvent "End: ValidateAgainstpVaccRefTable()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim recordset2 As ADODB.recordset
    Dim query As String
    Dim pmdValue As String
    Dim rowCount As Long
    Dim recordCount As Long
        
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM pvaccref WHERE ltrim(rtrim(vaccine)) == '" & aVaccine & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
'    If rowCount <> 1 Then
'        'technical error
'        RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
'                    VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
'    Else
        recordCount = recordset.Fields.Item(0)
        
        If recordCount < 1 Then
            If recordCount = 0 Then
                RaiseError VM_VACCINE_DOES_NOT_EXIST_NUMBER, _
                           VM_VACCINE_DOES_NOT_EXIST_MESSAGE
            Else
                'technical error
                RaiseError VM_VACC_REF_RECORD_COUNT_ERROR_NUMBER, _
                           VM_VACC_REF_RECORD_COUNT_ERROR_MESSAGE
                        
            End If
                
        End If
        
 '   End If
        
    CloseRecordSet recordset
                        
    
    If anActiveCheck = True Then
        If ValidateVaccineActiveStatus(aVaccine, aConnection) = False Then
            RaiseError VM_INACTIVE_VACCINE_NUMBER, _
                       VM_INACTIVE_VACCINE_MESSAGE
        End If
        
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateAgainstpVaccRefTable()", vbLogEventTypeInformation
    
    Exit Sub
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateAgainstpVaccRefTable() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Sub

'#########################################################################
' Function Name:    ValidateDuplicateDepotProviderOrder
'
' Description:      Validate the existence of the specified order
'
' Arguments:
'
' Returns:
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
'#########################################################################
Public Function ValidateDuplicateDepotProviderOrder( _
                                                anOrderDate As String, _
                                                aShipmentDate As String, _
                                                aDepotPIN As String, _
                                                aPIN As String, _
                                                aVaccine As String, _
                                                aLotNumber As String, _
                                                aConnection As ADODB.Connection) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDuplicateDepotProviderOrder()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim lotNumber As String
    lotNumber = Replace(aLotNumber, "-", "")
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM distrib "
    query = query & "WHERE  ltrim(rtrim(FromPin)) = '" & aDepotPIN & "' "
    query = query & "AND    ltrim(rtrim(pin)) ='" & aPIN & "' "
    query = query & "AND    ltrim(rtrim(vaccine))='" & aVaccine & "' "
    query = query & "AND    ltrim(rtrim(strtran(lot,ltrim(rtrim('-')),'')))='" & lotNumber & "' "
    ''' R1272 query = query & "AND    dateout=datevalue('" & aShipmentDate & "') "
    query = query & "AND    dateout={" & aShipmentDate & "} "
    query = query & "AND    action=1"
    Debug.Print query
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    Debug.Print rowCount
    
    If rowCount < 1 Then
         RaiseError VM_DUPLICATE_DEPOT_ORDER_ERROR_NUMBER, _
                    VM_DUPLICATE_DEPOT_ORDER_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
        Debug.Print recordCount
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_DEPOT_ORDER_ERROR_NUMBER, _
                       VM_DUPLICATE_DEPOT_ORDER_ERROR_MESSAGE
        ElseIf recordCount = 1 Then
            ValidateDuplicateDepotProviderOrder = True
        ElseIf recordCount = 0 Then
            ValidateDuplicateDepotProviderOrder = False
        Else
            RaiseError VM_DUPLICATE_DEPOT_ORDER_ERROR_NUMBER, _
                     VM_DUPLICATE_DEPOT_ORDER_ERROR_MESSAGE
        End If
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDuplicateDepotProviderOrder()", vbLogEventTypeInformation
    Exit Function
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDuplicateDepotProviderOrder() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Function

'#########################################################################
' Subroutine Name:  ValidateAvailableInventoryLot
'
' Description:      Validate the availablibity of state inventory of specific
'                   depot, vaccine and lot against the requested doses
'
' Arguments:
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
'                                         Removed the Format function.
'#########################################################################
Public Sub ValidateAvailableInventoryLot( _
                                            aDepotPIN As String, _
                                            aVaccine As String, _
                                            aLotNumber As String, _
                                            aDose As Long, _
                                            aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateAvailableInventoryLot()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim lotNumber As String
    
    lotNumber = Replace(aLotNumber, "-", "")
    
    Set recordset = CreateObject("ADODB.Recordset")
    query = "SELECT count(*) FROM Lots "
    query = query & "WHERE ltrim(rtrim(pin)) = '" & aDepotPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(strtran(lot,ltrim(rtrim('-')),''))) = '" & lotNumber & "' "
    query = query & "AND " & aDose & " <= Remain "
    ''' R1272 query = query & "AND Expiry >= " & " DateValue('" & CStr(Format(Date, "mm/dd/yyyy")) & "')"
    query = query & "AND Expiry >= " & " {" & CStr(Date) & "} "

    Debug.Print query
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    Debug.Print rowCount
    
    If rowCount < 1 Then
         RaiseError VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_NUMBER, _
                    VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
        Debug.Print recordCount
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_DUPLICATE_AVAIL_INVENTORY_LOT_NUMBER, _
                        VM_DUPLICATE_DUPLICATE_AVAIL_INVENTORY_LOT_MESSAGE
        ElseIf recordCount < 1 Then
            RaiseError VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_NUMBER, _
                        VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_MESSAGE
        End If
    End If
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateAvailableInventoryLot()", vbLogEventTypeInformation
    
    Exit Sub
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
     Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateAvailableInventoryLot() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub

'#########################################################################
' Subroutine Name:  ValidateNumberofDoses
'
' Description:      Validate the requested doses of the vaccine is a multiple
'                   of the vial size.  If the input dose is not a multiple of
'                   the vial size, the next higher multiple of the vial size will
'                   be assigned to the aDoseCount
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateNumberofDoses( _
                                    aDepotPIN As String, _
                                    aVaccine As String, _
                                    aLotNumber As String, _
                                    aDoseCount As Long, _
                                    aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateNumberofDoses()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim vialSize As Long
    Dim doseSize As Long
    Dim mutilple As Integer
    Dim lotNumber As String
    
    lotNumber = Replace(aLotNumber, "-", "")
    
    Set recordset = CreateObject("ADODB.Recordset")
    query = "SELECT size FROM Lots "
    query = query & "WHERE ltrim(rtrim(pin)) = '" & aDepotPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(strtran(lot,ltrim(rtrim('-')),''))) = '" & lotNumber & "' "
    Debug.Print query
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    Debug.Print rowCount
    
    If rowCount < 1 Then
         RaiseError VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_NUMBER, _
                    VM_AVAIL_INVENTORY_LOT_COUNT_ERROR_MESSAGE
    End If
    If IsNull(recordset.Fields.Item(0)) = False Then
        vialSize = recordset.Fields.Item(0)
    Else
        vialSize = 1
    End If
    
    doseSize = aDoseCount
    mutilple = doseSize \ vialSize
    If (vialSize > 0) Then
        If (doseSize Mod vialSize) > 0 Then
            doseSize = vialSize * (mutilple + 1)
            aDoseCount = doseSize
        End If
    End If
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateNumberofDoses()", vbLogEventTypeInformation
    
    Exit Sub
ErrorHandler:
   With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateNumberofDoses() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
                                    

End Sub
'#########################################################################
' Function Name: ValidateEnrollmentRecord
'
' Description: Function checks the enroll table to see if a record
' exists for the given provider pin
'
' Arguments:
'           aPIN -
'           aConnection - An open connection object to the database.
'
'Returns : True if record exists and False otherwise
'
' Exceptions:
'
'#########################################################################
Public Function ValidateEnrollmentRecord(aPIN As String, aConnection As ADODB.Connection) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateEnrollmentRecord()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Dim RetVal As Boolean
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM enroll WHERE ltrim(rtrim(pin)) = '" & aPIN & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_ENROLLMENT_DATA_RECORD_ERROR_NUMBER, _
                    VM_ENROLLMENT_DATA_RECORD_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount > 1 Then
        RaiseError VM_DUPLICATE_ENROLLMENT_DATA_RECORDS_NUMBER, _
                    VM_DUPLICATE_ENROLLMENT_DATA_RECORDS_MESSAGE
        
    ElseIf recordCount = 1 Then
        RetVal = True
    ElseIf recordCount = 0 Then
        RetVal = False
    Else
        RaiseError VM_ENROLLMENT_DATA_RECORD_ERROR_NUMBER, _
                    VM_ENROLLMENT_DATA_RECORD_ERROR_MESSAGE
    End If
    
    ValidateEnrollmentRecord = RetVal
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateEnrollmentRecord()", vbLogEventTypeInformation
    
Exit Function
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateEnrollmentRecord() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Subroutine Name: ValidateFundingCodeandPO
'
' Description: Subroutine verifies that the funding code
' must exist in funding table. If code is State (3),
' then the subroutine checks that the StatePO is not null. Exceptions
' are raised if there are duplicate entries in the funding table
' or if the funding code does not exist in the table or if the
' the funding code is State and the state po is null or is an empty
' string
'
' Arguments:
'           aFundingCode - The code for the funding source for the order.
'           aStatePO - The state purchase order number.
'           aConnection - An open database connection object.
'
' Exceptions:   VM_FUNDING_CODE_RECORD_COUNT_ERROR
'               VM_DUPLICATE_FUNDING_CODE
'               VM_UNKNOWN_FUNDING_CODE
'               VM_NULL_STATE_PO
'
'
'
'#########################################################################

Public Sub ValidateFundingCodeandPO( _
                                    aFundingCode As String, _
                                    aStatePO As String, _
                                    aConnection As ADODB.Connection)
 On Error GoTo ErrorHandler
     App.LogEvent "Begin: ValidateFundingCodeandPO()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim rowCount As Long
    Dim recordCount As Long
    Dim query As String
    
    Set recordset = CreateObject("ADODB.Recordset")
    query = "SELECT count(*) FROM funding WHERE ltrim(rtrim(fundcode)) = '" & aFundingCode & "'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount < 1 Then
         RaiseError VM_FUNDING_CODE_RECORD_COUNT_ERROR_NUMBER, _
                    VM_FUNDING_CODE_RECORD_COUNT_ERROR_MESSAGE
                    
    Else
        recordCount = recordset.Fields.Item(0)
        
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_FUNDING_CODE_NUMBER, _
                        VM_DUPLICATE_FUNDING_CODE_MESSAGE
                        
        ElseIf recordCount = 0 Then
             RaiseError VM_UNKNOWN_FUNDING_CODE_NUMBER, _
                        VM_UNKNOWN_FUNDING_CODE_MESSAGE
                        
        ElseIf recordCount < 0 Then
            RaiseError VM_FUNDING_CODE_RECORD_COUNT_ERROR_NUMBER, _
                     VM_FUNDING_CODE_RECORD_COUNT_ERROR_MESSAGE
                     
        End If
        
    End If
    
    If aFundingCode = VFC_FUNDING_CODE Or aFundingCode = G317_FUNDING_CODE Then
        
        If False = ValidateNullString(aStatePO) Then
            RaiseError VM_NO_STATE_PO_NEEDED_NUMBER, _
                        VM_NO_STATE_PO_NEEDED_MESSAGE
                        
        End If
    Else
        If True = ValidateNullString(aStatePO) Then
            RaiseError VM_NULL_STATE_PO_NUMBER, _
                       VM_NULL_STATE_PO_MESSAGE
                        
        End If
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
     App.LogEvent "End: ValidateFundingCodeandPO()", vbLogEventTypeInformation
     
 Exit Sub
 
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateFundingCodeandPO() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
    
End Sub



'#########################################################################
' Subroutine Name: ValidateActionCode
'
' Description: Subroutine checks if the action code given is
' a valid provider inventory adjustment in the actref table.
'
' Arguments:
'           anActionCode - Action code being validated.
'           aConnection - An open connection object to the database.
'
' Exceptions:   VM_ACTION_CODE_ERROR
'               VM_DUPLICATE_ACTION_CODE
'               VM_ACTION_CODE_ERROR
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1596       03/04/2002  Joe Bernard     ODBC 6.0 also does not work with
'                                         ltrim and rtrim on numeric or
'                                         date values. Removed these on
'                                         action field.
' R1701       06/20/2002  Joe Bernard     Dixmont (sub-request R1624)
'                                         Modified to include new action codes.
'#########################################################################

Public Sub ValidateActionCode(anActionCode As String, aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateActionCode()", vbLogEventTypeInformation
        
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
   
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM actref "
    query = query & "WHERE ltrim(rtrim(status)) = '" & anActionCode & "' "
    'R1596 query = query & "AND ltrim(rtrim(action)) BETWEEN " & RETURNED & " AND  " & TRANSFER
    'R1701 Commented out this line - query = query & "AND action BETWEEN " & RETURNED & " AND  " & TRANSFER  'R1596 - Changed from above
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount < 1 Then
         RaiseError VM_ACTION_CODE_ERROR_NUMBER, _
                    VM_ACTION_CODE_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount > 1 Then
        RaiseError VM_DUPLICATE_ACTION_CODES_NUMBER, _
        VM_DUPLICATE_ACTION_CODES_MESSAGE
        
    ElseIf recordCount < 1 Then
        RaiseError VM_ACTION_CODE_ERROR_NUMBER, _
        VM_ACTION_CODE_ERROR_MESSAGE
        
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateActionCode()", vbLogEventTypeInformation
Exit Sub
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateActionCode() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Sub

'#########################################################################
' Function Name: ValidateActiveValue
'
' Description: Checks whether anActiveValue is "Y" or "N"
'
' Arguments:
'           anActiveValue - Value being validated.
'
' Returns: True if it is and False otherwise
'
' Exceptions: None
'#########################################################################

Public Function ValidateActiveValue(anActiveValue As String) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateActiveValue()", vbLogEventTypeInformation
    
    If (StrComp("Y", UCase(anActiveValue)) = 0) Or (StrComp("N", UCase(anActiveValue)) = 0) Then
        ValidateActiveValue = True
    Else
        ValidateActiveValue = False
    End If
    App.LogEvent "End: ValidateActiveValue()", vbLogEventTypeInformation
      
Exit Function
    
ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateActiveValue() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Function

'#########################################################################
' Subroutine Name: ValidateAgainstVaccineContractTable
'
' Description: Subroutine checks whether an order defined by the
' input parameters in  the stock table satisfies the contract conditions.
' Specifically, the number of doses must be greater than some minimum,
' the number of doses ordered must be a multiple of some unit size, and
' the total cost of the vaccine must be less than $25,000.00
'
' Arguments:
'           aShipmentDate - The shipment date of the order.
'           aVaccine - Vaccine whose contract is being validated.
'           aDoseCount - Number of doses in the order.
'           aConnection - An open connection to the database
'
' Exceptions:   VM_CONTRACT_TABLE_ERROR_
'               VM_INVALID_CONTRACT
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
'#########################################################################
Public Sub ValidateAgainstVaccineContractTable( _
                                                aShipmentDate As String, _
                                                aVaccine As String, _
                                                aDoses As Long, _
                                                aPkgSize As Long, _
                                                aCost As String, _
                                                aConnection As ADODB.Connection)
 On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateAgainstVaccineContractTable()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Dim index As Long
    Dim unitsize As Long
    Dim notFound As Boolean
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT unitsize, pkgsize, cost FROM pvaccref "
    query = query & "WHERE minbulk <= " & aDoses & " "
    ''' R1272 query = query & "AND (datevalue('" & aShipmentDate & "') BETWEEN begin AND end) "
    query = query & "AND ({" & aShipmentDate & "} BETWEEN begin AND end) "
    query = query & "AND trim(vaccine) = '" & aVaccine & "' "
    query = query & "AND (" & aDoses & " * cost) <= " & TOTAL_COST
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount < 0 Then
         RaiseError VM_CONTRACT_TABLE_ERROR_NUMBER, _
                    VM_CONTRACT_TABLE_ERROR_MESSAGE
    End If
    
    notFound = True
    
    While (Not recordset.EOF) And (notFound = True)
        unitsize = recordset.Fields.Item(0)
        aPkgSize = recordset.Fields(1).Value
        aCost = recordset.Fields(2).Value
        If (aDoses Mod aPkgSize) = 0 Then
            notFound = False
        Else
            RaiseError VM_ORDER_DOSE_NOT_A_MULTIPLE_OF_VIAL_NUMBER, _
                        VM_ORDER_DOSE_NOT_A_MULTIPLE_OF_VIAL_MESSAGE
        End If
        recordset.MoveNext
    Wend
    
    If notFound = True Then
        RaiseError VM_INVALID_CONTRACT_NUMBER, _
                    VM_INVALID_CONTRACT_MESSAGE
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateAgainstVaccineContractTable()", vbLogEventTypeInformation

 Exit Sub
 
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.source
    Debug.Print Err.Description
    Debug.Print Err.Number
    Debug.Print Err.HelpFile
    Debug.Print Err.HelpContext
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateAgainstVaccineContractTable() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    Err.Raise Err
    
End Sub

'#########################################################################
' Function Name: ValidateDuplicateManufacturerOrder
'
' Description: Functions checks if  a record already exists
' in the stock table that is defined by the input parameters and
' that has not been sent to the CDC yet.
'
' Arguments:
'           anOrderDate - The reporting date of the order.
'           aPIN - The pin of the provider making the order.
'           aVaccine - The vaccine being ordered.
'           aFundingSource - The funding source for the order.
'           aConnection - An open connection to the database.
'
'
' Returns: True if it exists and False it does not.
'
' Exceptions:   VM_MANUFACTURER_ORDER_ERROR
'               VM_DUPLICATE_MANUFACTURER_ORDER
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
' R1596       03/05/2002  Joe Bernard     Corrected a problem where an error
'                         Stephen Nelson  occurred within validateEventDate...
'                                         The error handler assumed a recordset
'                                         existed, and was throwing an missing
'                                         object or with block error. Moved
'                                         the setting of the recordset to be
'                                         above the first validate function.
'                                         This is similar to the way all other
'                                         functions handle this.
' R1812:D4358       09/30/2004  Stephen Nelson  Manufacturer direct orders are
'                                         now processed through the NEWORDER
'                                         table, and not the STOCK table.
'                                         Made the necessary changes in
'                                         this procedure.
'                                         This change occurred with
'                                         the release of Vacman 3.0.8.
'                                         Also, cleaned up the old code
'                                         that had been commented out from
'                                         past releases.
'#########################################################################
Public Function ValidateDuplicateManufacturerOrder( _
                                                anOrderDate As String, _
                                                aPIN As String, _
                                                aVaccine As String, _
                                                aFundingSource As String, _
                                                aConnection As ADODB.Connection) As Boolean
 On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDuplicateManufacturerOrder()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Set recordset = CreateObject("ADODB.Recordset")  'R1596 - Moved this line here from below the following validate function
   
    ' Need to make sure that there is no order with the
    ' same parameters that has already been sent to the CDC.
    ' An exception is raised if such an order exists.
    ValidateEventDateManufacturerProviderOrder anOrderDate, _
                                                aPIN, _
                                                aVaccine, _
                                                aFundingSource, _
                                                aConnection
    
    query = "SELECT count(*) "
    query = query & "FROM neworder "  'R1812:D4358 Change
    query = query & "WHERE orddate = {" & anOrderDate & "} "
    query = query & "AND ltrim(rtrim(pin)) = '" & aPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(fcode1)) = '" & aFundingSource & "' "
    query = query & "AND seekid = 0"   ' R1812:D4358 Change
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_MANUFACTURER_ORDER_ERROR_NUMBER, _
                    VM_MANUFACTURER_ORDER_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount > 1 Then
        RaiseError VM_DUPLICATE_MANUFACTURER_ORDER_NUMBER, _
                        VM_DUPLICATE_MANUFACTURER_ORDER_MESSAGE
                        
    ElseIf recordCount = 1 Then
        ValidateDuplicateManufacturerOrder = True
        
    ElseIf recordCount = 0 Then
        ValidateDuplicateManufacturerOrder = False
        
    Else
        RaiseError VM_MANUFACTURER_ORDER_ERROR_NUMBER, _
                    VM_MANUFACTURER_ORDER_ERROR_MESSAGE
                    
    End If
        
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDuplicateManufacturerOrder()", vbLogEventTypeInformation

Exit Function
 
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDuplicateManufacturerOrder() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Function Name: ValidateDuplicateProviderInventoryAdjustment
'
' Description: Function checks a record already exists in the database
' that is defined by the input parameters
'
' Arguments:
'           aTransactionDate - The date that the adjustment was made by the provider.
'           aTransactionCode - The code for the adjustment.
'           aDepotPIN - The pin of the depot where the adjustment was reported.
'           aPIN - The pin of the provider who made the depot.
'           aVaccine - The vaccine in the adjustment.
'           aLotNumber - The lot number of the vaccine in the adjustment.
'           aConnection - An open connection to the database.
'
' Returns: True if it exists and False if it does not
'
' Exceptions:   VM_INVENTORY_ADJUSTMENT_RECORD_COUNT_ERROR
'               VM_DUPLICATE_INVENTORY_ADJUSTMENT
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
' R1596       03/04/2002  Joe Bernard     ODBC 6.0 also does not work with
'                                         ltrim and rtrim on numeric or
'                                         date values. Removed these on action
'                                         and dateout fields.
'#########################################################################
Public Function ValidateDuplicateProviderInventoryAdjustment( _
                                                        aTransactionDate As String, _
                                                        aTransactionCode As Long, _
                                                        aDepotPIN As String, _
                                                        aPIN As String, _
                                                        aVaccine As String, _
                                                        aLotNumber As String, _
                                                        aConnection As ADODB.Connection) As Boolean

On Error GoTo ErrorHandler:
    App.LogEvent "Begin: ValidateDuplicateProviderInventoryAdjustment()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Dim lotNumber As String
   
    lotNumber = Replace(aLotNumber, "-", "")
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) "
    query = query & "FROM distrib "
    'R1596 query = query & "WHERE ltrim(rtrim(action)) = " & aTransactionCode & " "
    query = query & "WHERE action = " & aTransactionCode & " "  'R1596 - Changed from above
    query = query & "AND ltrim(rtrim(pin)) = '" & aPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(strtran(lot,ltrim(rtrim('-')),''))) = '" & lotNumber & "' "
    ''' R1272 query = query & "AND ltrim(rtrim(dateout)) = DateValue('" & aTransactionDate & "') "
    'R1596 query = query & "AND ltrim(rtrim(dateout)) = {" & aTransactionDate & "} "
    query = query & "AND dateout = {" & aTransactionDate & "} "  'R1596 - Changed from above
    query = query & "AND ltrim(rtrim(frompin)) = '" & aDepotPIN & "'"
    
    Debug.Print query
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount < 1 Then
         RaiseError VM_INVENTORY_ADJUSTMENT_RECORD_COUNT_ERROR_NUMBER, _
                    VM_INVENTORY_ADJUSTMENT_RECORD_COUNT_ERROR_MESSAGE
                    
    Else
        recordCount = recordset.Fields.Item(0)
    
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_INVENTORY_ADJUSTMENT_NUMBER, _
                        VM_DUPLICATE_INVENTORY_ADJUSTMENT_MESSAGE
                        
        ElseIf recordCount = 1 Then
            ValidateDuplicateProviderInventoryAdjustment = True
            
        ElseIf recordCount = 0 Then
            ValidateDuplicateProviderInventoryAdjustment = False
            
        Else
            RaiseError VM_INVENTORY_ADJUSTMENT_RECORD_COUNT_ERROR_NUMBER, _
                     VM_INVENTORY_ADJUSTMENT_RECORD_COUNT_ERROR_MESSAGE
                     
        End If
        
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDuplicateProviderInventoryAdjustment()", vbLogEventTypeInformation
Exit Function
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDuplicateProviderInventoryAdjustment() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Function

'#########################################################################
' Subroutine Name: ValidateEventDateManufacturerProviderOrder
'
' Description: The subroutine checks if a manufacturer order that is defined
' by the input parameters has already been sent to the CDC. An exception is raised
' if the order has been sent to the CDC. This exception means that the record
' can not be updated and a new record with the same parameters cannot be created.
'
' Arguments:
'           anOrderDate - The reporting date of the order.
'           aPIN - The pin of the provider making the order.
'           aVaccine - The vaccine being ordered.
'           aFundingSource - The funding source for the order.
'           aConnection - An open connection to the database.
'
' Exceptions:
'               VM_EVENT_DATE_ERROR
'               VM_DUPLICATE_MANUFACTURER
'               VM_MANUFACTURER_ORDER_AT_CDC
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
'                                         xmitdate is a date/time value,
'                                         removed the trim functions.
' R1812:D4358       09/30/2004  Stephen Nelson  Manufacturer direct orders are
'                                         now processed through the NEWORDER
'                                         table, and not the STOCK table.
'                                         Made the necessary changes in
'                                         this procedure.
'                                         This change occurred with
'                                         the release of Vacman 3.0.8.
'                                         Also, cleaned up the old code
'                                         that had been commented out from
'                                         past releases.
'#########################################################################
Public Sub ValidateEventDateManufacturerProviderOrder( _
                                                        anOrderDate As String, _
                                                        aPIN As String, _
                                                        aVaccine As String, _
                                                        aFundingSource As String, _
                                                        aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateEventDateManufacturerProviderOrder()", vbLogEventTypeInformation
    
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
     
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) "
    query = query & "FROM neworder "   ' R1812:D4358 Change
    query = query & "WHERE orddate = {" & anOrderDate & "} "
    query = query & "AND ltrim(rtrim(pin)) = '" & aPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(fcode1)) = '" & aFundingSource & "' "
    query = query & "AND seekid <> 0"      ' R1812:D4358 Change
        
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_EVENT_DATE_ERROR_NUMBER, _
                    VM_EVENT_DATE_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
        
    If recordCount <> 0 Then
            
        If recordCount > 1 Then
            RaiseError VM_DUPLICATE_MANUFACTURER_ORDER_NUMBER, _
                        VM_DUPLICATE_MANUFACTURER_ORDER_MESSAGE
                        
        ElseIf recordCount = 1 Then
            RaiseError VM_MANUFACTURER_ORDER_AT_CDC_NUMBER, _
                    VM_MANUFACTURER_ORDER_AT_CDC_MESSAGE
        
        Else
            RaiseError VM_EVENT_DATE_ERROR_NUMBER, _
                    VM_EVENT_DATE_ERROR_MESSAGE
                                        
        End If

    End If
         
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateEventDateManufacturerProviderOrder()", vbLogEventTypeInformation
Exit Sub
 
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateEventDateManufacturerProviderOrder() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub

'#########################################################################
' Function Name: ValidatePhoneFaxFormat
'
' Description:  Validates whether the string given is in the format
'               (xxx)xxx-xxxx. Returns True is valid and False
'               otherwise.
'
' Arguments: aPhoneFax
'
' Exceptions:   None
'#########################################################################

Public Function ValidatePhoneFaxFormat( _
                                    aPhoneFax As String) As Boolean

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidatePhoneFaxFormat()", vbLogEventTypeInformation
    
    Dim RetVal As Boolean
    Dim normalFormatString As String
    Dim tempNumber As String
    
    If (Len(aPhoneFax) <> PHONE_FAX_NUMBER_LENGTH) Then
        RetVal = False
    Else
        
        'Strip the number of all tabs, spaces, parenthesis and dashes
        tempNumber = stripString(Trim(aPhoneFax))
        
        normalFormatString = Format(tempNumber, PHONE_FAX_FORMAT)
    
        If StrComp(normalFormatString, aPhoneFax) = 0 Then
            RetVal = True
        Else
            RetVal = False
        End If
    End If
    
    ValidatePhoneFaxFormat = RetVal
    App.LogEvent "End: ValidatePhoneFaxFormat()", vbLogEventTypeInformation
    
Exit Function

ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidatePhoneFaxFormat() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Subroutine Name: ValidateProviderInventoryAdjustDates
'
' Description: The subroutine checks to make sure that the transaction
' date of the adjustment is on or before the entry date.
'
' Arguments:
'           anEntryDate - The date the adjustment is being reported.
'           aTransactionDate - The date the adjustment was made by the provider.
'
' Exceptions: VM_INVENTORY_ADJUSTMENT_DATE_ERROR
'               VM_INVALID_ADJUSTMENT_ENTRY_DATE
'               VM_INVALID_ADJUSTMENT_TRANSACTION_DATE
'
'#########################################################################

Public Sub ValidateProviderInventoryAdjustDates( _
                                                anEntryDate As String, _
                                                aTransactionDate As String)

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateProviderInventoryAdjustDates()", vbLogEventTypeInformation
    Dim entryDate As Date
    Dim transactionDate As Date
    Dim currentDate As Date
    
    'First make sure that the input dates can be converted to dates
    
    If IsDate(anEntryDate) = False Then
        RaiseError VM_INVALID_ADJUSTMENT_ENTRY_DATE_NUMBER, _
                    VM_INVALID_ADJUSTMENT_ENTRY_DATE_MESSAGE
    End If
    
    If IsDate(aTransactionDate) = False Then
        RaiseError VM_INVALID_ADJUSTMENT_TRANSACTION_DATE_NUMBER, _
                    VM_INVALID_ADJUSTMENT_TRANSACTION_DATE_MESSAGE
    End If
    
    entryDate = DateValue(anEntryDate)
    transactionDate = DateValue(aTransactionDate)
   
    currentDate = Date
    
  ' MPF - 4/25/2007 Comment out two checks to allow for future dates
    'If entryDate > currentDate Then
    '    RaiseError VM_INVENTORY_ADJUSTMENT_ENTRY_DATE_NUMBER, _
    '                VM_INVENTORY_ADJUSTMENT_ENTRY_DATE_MESSAGE
    'End If
    
    'If transactionDate > entryDate Then
    '    RaiseError VM_INVENTORY_ADJUSTMENT_DATE_ERROR_NUMBER, _
    '                VM_INVENTORY_ADJUSTMENT_DATE_ERROR_MESSAGE
    'End If
    
    App.LogEvent "End: ValidateProviderInventoryAdjustDates()", vbLogEventTypeInformation
Exit Sub

ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateProviderInventoryAdjustDates() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
    
End Sub

'#########################################################################
' Subroutine Name: ValidateSector
'
' Description:
'
' Arguments:
'           aSector
'           aConnection
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  Sector is a number value. Removed
'                                         the trim functions.
'#########################################################################
Public Function ValidateSector( _
                            aSector As Long, _
                            aConnection As ADODB.Connection)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateSector()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
   
    Set recordset = CreateObject("ADODB.Recordset")
    query = "SELECT count(*) FROM sectoref "
    query = query & "WHERE sector = " & aSector & " "
    ''' R1272 query = query & "AND trim(sector) BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX
    query = query & "AND sector BETWEEN " & PROVIDER_SECTOR_MIN & " AND " & PROVIDER_SECTOR_MAX
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_SECTOR_ERROR_NUMBER, _
                    VM_SECTOR_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount > 1 Then
        RaiseError VM_DUPLICATE_SECTOR_NUMBER, _
        VM_DUPLICATE_SECTOR_MESSAGE
        
    ElseIf recordCount = 1 Then
        ValidateSector = True
        
    ElseIf recordCount = 0 Then
        ValidateSector = False
        
    Else
        RaiseError VM_SECTOR_ERROR_NUMBER, _
            VM_SECTOR_ERROR_MESSAGE
            
    End If

    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateSector()", vbLogEventTypeInformation
Exit Function

ErrorHandler:
   With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateSector() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
                            

End Function

'#########################################################################
' Subroutine Name:  ValidateVaccAdminSector
'
' Description:      Validate that the given sector for doses administered
' is 1 or 2 for public and private respectively
'
' Arguments: aSector
'
' Exceptions: VM_INVALID_VACC_ADMIN_SECTOR
'#########################################################################

Public Sub ValidateVaccAdminSector(aSector As String)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateVaccAdminSector()", vbLogEventTypeInformation
    
    If (aSector <> VACC_ADMIN_PUBLIC_SECTOR) And (aSector <> VACC_ADMIN_PRIVATE_SECTOR) Then
        RaiseError VM_INVALID_VACC_ADMIN_SECTOR_NUMBER, _
                    VM_INVALID_VACC_ADMIN_SECTOR_MESSAGE
    End If
    App.LogEvent "End: ValidateVaccAdminSector()", vbLogEventTypeInformation
Exit Sub
ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateVaccAdminSector() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Sub

'#########################################################################
' Subroutine Name:  ValidateVaccineLot
'
' Description:      Validate the existence of the state vaccine lot
'
' Arguments:
'
' Exceptions:
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' IMMPACT-800 10/23/2007  Matt Martin     Strip dash character from
'                                         lot number comparisons
'#########################################################################

Public Sub ValidateVaccineLot( _
                                aVaccine As String, _
                                aLotNumber As String, _
                                aConnection As ADODB.Connection)

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateVaccineLot()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Dim lotNumber As String
    
    lotNumber = Replace(aLotNumber, "-", "")
    
    Set recordset = CreateObject("ADODB.Recordset")
    query = "SELECT count(*) FROM stock "
    query = query & "WHERE ltrim(rtrim(vaccine)) = '" & aVaccine & "' "
    query = query & "AND ltrim(rtrim(strtran(lot,ltrim(rtrim('-')),''))) = '" & lotNumber & "'"
    
    Debug.Print query
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    Debug.Print rowCount
    
    If rowCount < 1 Then
         RaiseError VM_LOT_NUMBER_ERROR_NUMBER, _
                    VM_LOT_NUMBER_ERROR_MESSAGE
    End If
    recordCount = recordset.Fields.Item(0)
    
    'If recordCount > 1 Then
    '   RaiseError VM_DUPLICATE_LOT_NUMBER_NUMBER, _
    '  VM_DUPLICATE_LOT_NUMBER_MESSAGE
    'End If
    If recordCount < 1 Then
        RaiseError VM_LOT_NUMBER_ERROR_NUMBER, _
        VM_LOT_NUMBER_ERROR_MESSAGE
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateVaccineLot()", vbLogEventTypeInformation

Exit Sub

ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateVaccineLot() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub

'#########################################################################
' Subroutine Name: ValidateVFCCertDate
'
' Description: the subroutine makes sure that the VFC certification date
' is a valid date.
'
' Arguments: aVFCCertificationDate
'
' Exceptions:VM_INVALID_VFC_CERT_DATE
'
'#########################################################################

Public Sub ValidateVFCCertDate( _
                                aVFCCertificationDate As String)

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateVFCCertDate()", vbLogEventTypeInformation
    If IsDate(aVFCCertificationDate) = False Then
        RaiseError VM_INVALID_VFC_CERT_DATE_NUMBER, _
                VM_INVALID_VFC_CERT_DATE_MESSAGE
    End If
    App.LogEvent "End: ValidateVFCCertDate()", vbLogEventTypeInformation

Exit Sub

ErrorHandler:
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateVFCCertDate() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub

'#########################################################################
' Function Name:    ValidateDuplicateDosesAdmin
'
' Description:      Validate the existence of the dose admin record for a given
'                   report date, pin, vaccine, dose series
'
' Arguments:
'
' Returns:
'
' Exceptions:
'
' CHANGE HISTORY
' Request Id  Date        Author          Description
' ------------------------------------------------------------------------
' R1352       11/05/2001  Stephen Nelson  ODBC 6.0 does not work with the
'                                         DateValue() function.  Replaced
'                                         with brackets {}.
'                                         Sector is a number value. Removed
'                                         the trim functions
' R1596       03/04/2002  Joe Bernard     ODBC 6.0 also does not work with
'                                         ltrim and rtrim on numeric or
'                                         date values. Removed these on dose
'                                         and adate fields. Also corrected
'                                         problem with single quotes for
'                                         values compared or assigned to
'                                         the dose and sector fields.
' R1812:D4358 09/30/2004  Stephen Nelson  Release Tulane
'                                         Removed the leading spaces from
'                                         the PIN value.
'                                         This change occurred with
'                                         the release of Vacman 3.0.8.
'                                         Also, cleaned up the old code
'                                         that had been commented out from
'                                         past releases.
'#########################################################################
Public Function ValidateDuplicateDosesAdmin( _
                                            aReportingDate As String, _
                                            aPIN As String, _
                                            aVaccine As String, _
                                            aDosesInSeries As String, _
                                            aSector As String, _
                                            aConnection As ADODB.Connection) As Boolean
                                                
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateDuplicateDosesAdmin()", vbLogEventTypeInformation
    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Set recordset = CreateObject("ADODB.Recordset")
    
    ' R1812:D4358 Start Change
    query = "SELECT count(*) "
    query = query & "FROM DosesAdm "
    query = query & "WHERE ltrim(rtrim(pin)) = '" & aPIN & "' "
    query = query & "AND ltrim(rtrim(vaccine))='" & aVaccine & "' "
    query = query & "AND dose = " & aDosesInSeries & " "
    query = query & "AND sector = " & aSector & " "
    query = query & "AND adate ={" & aReportingDate & "} "
    ' R1812:D4358 End Change

    Debug.Print query
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    rowCount = recordset.recordCount
    Debug.Print rowCount
    
    If rowCount < 1 Then
         RaiseError VM_DUPLICATE_DOSES_ADMIN_ERROR_NUMBER, _
                    VM_DUPLICATE_DOSES_ADMIN_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
        Debug.Print recordCount
        If recordCount > 1 Then
           RaiseError VM_DUPLICATE_DOSES_ADMIN_ERROR_NUMBER, _
                       VM_DUPLICATE_DOSES_ADMIN_ERROR_MESSAGE
        ElseIf recordCount = 1 Then
            ValidateDuplicateDosesAdmin = True
        ElseIf recordCount = 0 Then
            ValidateDuplicateDosesAdmin = False
        Else
            RaiseError VM_DUPLICATE_DOSES_ADMIN_ERROR_NUMBER, _
                     VM_DUPLICATE_DOSES_ADMIN_ERROR_MESSAGE
        End If
    End If
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateDuplicateDosesAdmin()", vbLogEventTypeInformation
    Exit Function
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateDuplicateDosesAdmin() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
End Function

'#########################################################################
' Subroutine Name:  ValidateTotalDoses
'
' Description:      Validate the total doses equal to the distributions
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateTotalDoses( _
                                Optional aDosesLessThanOne As Long = 0, _
                                Optional aDosesForOneYear As Long = 0, _
                                Optional aDosesForTwoYears As Long = 0, _
                                Optional aDosesForThreetoFourYears As Long = 0, _
                                Optional aDosesForFiveYears As Long = 0, _
                                Optional aDosesForSixtoNineYears As Long = 0, _
                                Optional aDosesForTentoFourteenYears As Long = 0, _
                                Optional aDosesForFifteentoNineteenYears As Long = 0, _
                                Optional aDosesForTwentytoTwentyFourYears As Long = 0, _
                                Optional aDosesForTwentyFivetoFourtyFourYears As Long = 0, _
                                Optional aDosesForFourtyFivetoSixtyFourYears As Long = 0, _
                                Optional aDosesForGreaterThanSixtyFiveYears As Long = 0, _
                                Optional anUnknown As Long = 0, _
                                Optional aTotalDoses As Long = 0)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateTotalDoses()", vbLogEventTypeInformation
    Dim qty As Long
    
    qty = aDosesLessThanOne + aDosesForOneYear + aDosesForTwoYears + aDosesForThreetoFourYears
    qty = qty + aDosesForFiveYears + aDosesForSixtoNineYears + aDosesForTentoFourteenYears
    qty = qty + aDosesForFifteentoNineteenYears
    qty = qty + aDosesForTwentytoTwentyFourYears + aDosesForTwentyFivetoFourtyFourYears
    qty = qty + aDosesForFourtyFivetoSixtyFourYears + aDosesForGreaterThanSixtyFiveYears
    qty = qty + anUnknown
    
    If qty <> aTotalDoses Then
        RaiseError VM_DIST_TOTAL_DISMATCH_NUMBER, _
                    VM_DIST_TOTAL_DISMATCH_MESSAGE
    End If
    App.LogEvent "End: ValidateTotalDoses()", vbLogEventTypeInformation
    Exit Sub
ErrorHandler:
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateTotalDoses() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub
'#########################################################################
' Subroutine Name:  ValidateExcessAllowance
'
' Description:      Validate the excess allowance is between 1 to 5
'
' Arguments:
'
' Exceptions:
'#########################################################################

Public Sub ValidateExcessAllowance(anExcessAllowance As String)
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateExcessAllowance()", vbLogEventTypeInformation
    Dim excess As Long
    excess = CLng(anExcessAllowance)
    If ((excess < EXCESS_ALLOW_MIN) Or (excess > EXCESS_ALLOW_MAX)) Then
        RaiseError VM_EXCESS_ALLOW_RANGE_ERROR_NUMBER, _
                    VM_EXCESS_ALLOW_RANGE_ERROR_MESSAGE
    End If
    Exit Sub
    App.LogEvent "End: ValidateExcessAllowance()", vbLogEventTypeInformation
ErrorHandler:
    Debug.Print Err.Number
    Debug.Print Err.Description
    Debug.Print Err.source
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateExcessAllowance() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc

End Sub

'#########################################################################
' Function Name: ValidateProvideraVaricellaEligibility
'
' Description: return the varicella eligilibity of the provider
'
' Arguments:
'       aPIN - The pin of the provider being validated.
'       aConnection - An open connection object.
'
' Returns: return TRUE if the provider is varicella eligilible FALSE otherwise
'
'
'#########################################################################

Public Function ValidateProvideraVaricellaEligibility(aPIN As String, _
                                                    aConnection As ADODB.Connection) As Boolean
On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidateProvideraVaricellaEligibility()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim recordCount As Long
    Dim rowCount As Long
    Dim PIN As String
    
    
    Set recordset = CreateObject("ADODB.Recordset")

    PIN = UCase(aPIN)
        
    If Len(PIN) > MAX_PIN_SIZE Then
        RaiseError VM_MAX_PIN_SIZE_VIOLATION_NUMBER, _
                   VM_MAX_PIN_SIZE_VIOLATION_MESSAGE
        
    End If
    
    
    If IsAlphaNumeric(PIN) = False Then
        RaiseError VM_INVALID_PROVIDER_PIN_TYPE_NUMBER, _
                    VM_INVALID_PROVIDER_PIN_TYPE_MESSAGE
    End If
    
    
    If True = ValidateNullString(PIN) Then
        RaiseError VM_NULL_PROVIDER_PIN_NUMBER, _
                VM_NULL_PROVIDER_PIN_MESSAGE
    End If
            
 
    query = "SELECT count(*) FROM provider WHERE ltrim(rtrim(pin)) = '" & PIN & "'"
    query = query & " and trim(codes)='V'"
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
        RaiseError VM_PROVIDER_PIN_RECORD_COUNT_ERROR_NUMBER, _
                   VM_PROVIDER_PIN_RECORD_COUNT_ERROR_MESSAGE
    Else
        recordCount = recordset.Fields.Item(0)
        
        If recordCount = 0 Then
             ValidateProvideraVaricellaEligibility = False
             
        ElseIf recordCount = 1 Then
            ValidateProvideraVaricellaEligibility = True
    
        ElseIf recordCount > 1 Then
            RaiseError VM_DUPLICATE_PROVIDER_PINS_NUMBER, _
                       VM_DUPLICATE_PROVIDER_PINS_MESSAGE
        
        Else
            RaiseError VM_PROVIDER_PIN_RECORD_COUNT_ERROR_NUMBER, _
                    VM_PROVIDER_PIN_RECORD_COUNT_ERROR_MESSAGE
        
        End If
    
    End If
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidateProvideraVaricellaEligibility()", vbLogEventTypeInformation
      
    Exit Function
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidateProvideraVaricellaEligibility() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

'#########################################################################
' Function Name: ValidatePhysicnRecord
'
' Description: Function checks the physicn table to see if a record
' exists for the given provider pin
'
' Arguments:
'           aPIN -
'           aConnection - An open connection object to the database.
'
'Returns : True if record exists and False otherwise
'
' Exceptions:
'
'#########################################################################
Public Function ValidatePhysicnRecord(aPIN As String, aLastName As String, aFirstName As String, aConnection As ADODB.Connection) As Boolean

On Error GoTo ErrorHandler
    App.LogEvent "Begin: ValidatePhysicnRecord()", vbLogEventTypeInformation

    Dim recordset As ADODB.recordset
    Dim query As String
    Dim rowCount As Long
    Dim recordCount As Long
    Dim RetVal As Boolean
    
    Set recordset = CreateObject("ADODB.Recordset")
    
    query = "SELECT count(*) FROM physicn WHERE pin = '" & aPIN & "' AND "
    query = query & "lastname = '" & aLastName & "' AND firstname = '" & aFirstName & "' "
    
    recordset.Open query, aConnection, adOpenKeyset, adLockReadOnly, adCmdText
    
    rowCount = recordset.recordCount
    
    If rowCount <> 1 Then
         RaiseError VM_PHYSICN_DATA_RECORD_ERROR_NUMBER, _
                    VM_PHYSICN_DATA_RECORD_ERROR_MESSAGE
    End If
    
    recordCount = recordset.Fields.Item(0)
    
    If recordCount > 1 Then
        RaiseError VM_DUPLICATE_PHYSICN_DATA_RECORDS_NUMBER, _
                    VM_DUPLICATE_PHYSICN_DATA_RECORDS_MESSAGE
        
    ElseIf recordCount = 1 Then
        RetVal = True
    ElseIf recordCount = 0 Then
        RetVal = False
    Else
        RaiseError VM_PHYSICN_DATA_RECORD_ERROR_NUMBER, _
                    VM_PHYSICN_DATA_RECORD_ERROR_MESSAGE
    End If
    
    ValidatePhysicnRecord = RetVal
    
    CloseRecordSet recordset
    Set recordset = Nothing
    App.LogEvent "End: ValidatePhysicnRecord()", vbLogEventTypeInformation
    
Exit Function
    
ErrorHandler:
    With recordset
        If .state = adStateOpen Then
            .Close
        End If
    End With
    
    Set recordset = Nothing
    Dim desc As String
    Dim code As Long
    desc = Err.Description
    code = Err.Number
    App.LogEvent "Problem: In ValidatePhysicnRecord() " & desc, vbLogEventTypeError
    Err.Raise code, "VACMAN DLL", desc
    
End Function

